#!/bin/bash
BuildDir=$1
BuildFloder=$BuildDir/dbus_proxy_addon
echo "Build output dir: " $BuildDir

if [ -e $BuildFloder ]; then
    rm -rf $BuildFloder
fi

mkdir $BuildFloder
ROOT=$(dirname $(readlink -f "$0"))
cp -rf $ROOT/* $BuildFloder

cd $BuildFloder
# echo "addon root:"  $ROOT

# 删除includes和libs，如果存在的话
if [ -e $BuildFloder/includes ]; then
    rm -rf $BuildFloder/includes
fi

mkdir -p $BuildFloder/includes

if [ -e $BuildFloder/libs ]; then
    rm -rf /$BuildFloder/libs/
fi

mkdir -p $BuildFloder/libs/

#编译主机CPU的个数
echo "BUILD_CPU_NUM = ${BUILD_CPU_NUM}"

if [ ! $BUILD_CPU_NUM ]; then
BUILD_CPU_NUM = `cat /proc/cpuinfo| grep "processor"| wc -l`
fi



# build jsoncpp
JSONCPP_PATH=$BuildFloder/deps/jsoncpp
echo "start building jsoncpp"
cd $JSONCPP_PATH
if [ -e build ]; then
    rm -rf build/
fi

mkdir -p build/debug
cd build/debug
cmake -DCMAKE_BUILD_TYPE=debug -DBUILD_STATIC_LIBS=ON -DBUILD_SHARED_LIBS=ON -DCMAKE_TOOLCHAIN_FILE=${CMAKE_TOOLCHAIN_FILE} -DARCHIVE_INSTALL_DIR=. -G "Unix Makefiles" ../..
make VERBOSE=1 -j$BUILD_CPU_NUM

cd $JSONCPP_PATH

LIB_JSONCPP=build/debug/src/lib_json/libjsoncpp_static.a

if [ -e $LIB_JSONCPP ]
then
    cp $LIB_JSONCPP $BuildFloder/libs/
    cp -r include/json $BuildFloder/includes/
else
    echo "compile Jsoncpp failed. exit."
    exit 1
fi
# # build libuv
echo "start building libuv..."
UV_PATH=$BuildFloder/deps/libuv
cd $UV_PATH
sh autogen.sh

echo "${CFLAGS} -fPIC CC=$CC CPP=$CC --host=$ARCH-linux"

./configure CFLAGS="${CFLAGS} -fPIC" CC=$CC  --host=$ARCH-linux
make VERBOSE=1 -j$BUILD_CPU_NUM

cd $UV_PATH

LIB_UV=.libs/libuv.a
if [ -e $LIB_UV ]
then
    cp $LIB_UV $BuildFloder/libs/
    cp -r include $BuildFloder/includes/uv
else
    echo "compile libuv failed. exit."
    exit 1
fi
make clean

# build dbus
echo "start building libdbus..."
DBUS_PATH=$BuildFloder/deps/dbus-1.10.18

cd $DBUS_PATH

make clean

./configure \
     CFLAGS="${CFLAGS} -fPIC -I${PRODUCT_ROOT}/${BOARD}/prebuilt/include" \
     LDFLAGS="-L${PRODUCT_ROOT}/${BOARD}/prebuilt/lib" \
     --disable-tests \
     --disable-xml-docs \
     --disable-modular-tests \
     --disable-systemd \
     --disable-inotify \
     --disable-doxygen-docs \
     --enable-abstract-sockets \
     --host=$ARCH-linux

make VERBOSE=1 -j$BUILD_CPU_NUM

cd $DBUS_PATH

LIB_DBUS=dbus/.libs/libdbus-1.a

if [ -e $LIB_DBUS ]
then
    cp $LIB_DBUS $BuildFloder/libs/
    rm -rf $BuildFloder/includes/dbus
    mkdir -p $BuildFloder/includes/dbus
    cp dbus/*.h $BuildFloder/includes/dbus/
else
    echo "compile libdbus failed. exit."
    exit 1
fi
# 清除掉编译产生的临时文件，否则会很多临时文件
#make clean
echo "compling dependens done."

cd $BuildFloder

source $BUILD_OUTPUT_ROOT/setenv-node.sh
#node-gyp configure && node-gyp build

node-gyp clean configure  rebuild --verbose  $BUILD_TYPE --arch=$ARCH
#npm 安装有问题
#npm install ./ -g

cd $BuildFloder

T_ADDON=$BuildFloder/build/Release/dbus_wrap.node
S_ADDON=$ROOT/../dbus_wrap.node

# if [ -L $S_ADDON ]; then
#     rm $S_ADDON
# fi
# 建立软链接到目的文件
#ln -s $T_ADDON $S_ADDON
rm -rf $S_ADDON
cp $T_ADDON $S_ADDON

rm -rf $BuildFloder

echo "==== iot-gravity addon compiled successed. ====="
