/*
 * Copyright (c) 2018 Alibaba Group Holding Ltd. All rights reserved.
 */

'use strict';

const Log = require('../../../utils/log');
const constant = require('../../../utils/constant');
const SystemFunction = require('./index');
const DBusProxy = require('../../dbus/dbus-proxy/DBusProxy');

const TAG = '[Function-Router]';

const TOPIC_PATTERN = /^\/[\w-@:\.]+(\/[\w-@:\.]+)*$/;

class Router extends SystemFunction {
  constructor(functionService) {
    super(functionService);
    this.id = constant.ROUTER_FUNCTION_ID;
  }

  handler(event, context, callback) {
    Log.i(TAG, `Handler runs with "${event}", "${JSON.stringify(context)}."`);
    var source = context.invokerContext.custom.source;
    var topic = context.invokerContext.custom.topic;
    if (source === undefined || source === null) {
      callback(new Error(`Source is not found in InvokerContext.`));
      return;
    }
    if (topic === undefined || topic === null) {
      callback(new Error(`Topic is not found in InvokerContext.`));
      return;
    }
    if (!TOPIC_PATTERN.test(topic)) {
      callback(new Error(`Topic contains illegal character. Allowed pattern is ${TOPIC_PATTERN}.`));
      return;
    }
    if (event === undefined || event === null) {
      callback(new Error(`Payload is required: "${event}".`));
      return;
    }
    var payload = event.toString();
    var isLongLived = this.functionService.isLongLivedFunction(source);
    var iface = isLongLived ? `iot.fc.FCLong_${source}` : `iot.fc.FCShort_${source}`;
    DBusProxy.proxy.sendTargetSignal(constant.FILTER_BUS_NAME, topic, iface, 'fc_event',
      payload);
    callback(null);
  }
}

module.exports = Router;

