const util = require('util');
const fs = require('fs');
const path = require('path');
const Action = require('./actions.js');
const log = require('../developer/log');
const constant = require("../../utils/constant");
const getUUID = require("./getUUID");
const TAG = "runner_ifttt";

const ExecStatus = {
    success: "execute ifttt success.",
    failed: "execute ifttt failed.",
    format_error: "ifttt format error.",
    read_file_error: "read file error."
};

function Ifttt(jobInfo, proxy) {
    this.jobInfo = jobInfo;
    this.proxy = proxy;
    this.traceId = getUUID();
};


Ifttt.prototype.checkFormat = function (sceneObj) {
    var formatted = false;
    var hasId = sceneObj[constant.ID] || false;
    var hasAction = sceneObj[constant.ACTION] || false;
    if (hasAction && hasId) {
        this.sceneid = sceneObj[constant.ID];
        formatted = (sceneObj[constant.ACTION].length > 0);
    }
    return formatted && hasId && hasAction;
};

Ifttt.prototype.sendJobStatus = function (err, result, startTime, detailsLog, callback) {
    /*
     *  {
     *      "ackCode" : int
     *      "jobId" :   long
     *      "jobStatus" : int // 0: 删除成功， 2:运行中,  4:执行失败
     *      "executor" : string
     *      "result" : string
     *  }
     */
    var info = this.jobInfo;
    var response = {};
    response.ackCode = info.ackCode;
    response.jobId = info.jobId;
    response.executor = info.executor;
    // response.jobStatus = err ? 4 : 2;
    response.jobStatus = 2;
    response.category = 0;
    response.sceneId = info.sceneId;
    var tag = '[IFTTT_Running_Result]';
    if (!err) {
        tag += '[Succeeded]';
    } else {
        tag += '[Failed]';
    }
    var isSuccess = err ? false : true ;
    log.LOGI(TAG, "jobId" + info.jobId + " result = " + result.toString());
    var useTime = new Date() - startTime;
    var logIFTTT = this.generateLog(useTime, isSuccess);
    response.result = logIFTTT + detailsLog;
    /*response.msg = {};
    response.msg.code = 0;
    response.msg.detail = logIFTTT + detailsLog;*/
    log.LOGM(tag, '[' + info.sceneId + ']', 'jobid', response.jobId, 'running status :', response.jobStatus, 'result :', response.result);
    this.proxy.sendJobExecuteStatus(info.jobId, response, function (err, status) {
        callback(err, status);
    });
};

/**
 *  keys = ['tenantId', 'ownerType', 'ownerId', 'bizCode', 'traceId', 'sceneId', 'useTime', 'result', 'timestamp'];
 * @param useTime
 * @param result
 */
Ifttt.prototype.generateLog = function (useTime, isSuccess) {
    var logSummary = util.format("[%s]", (new Date).toLocaleString());
    if(this.config && (this.config)[constant.SCENEOWNER]) {
        try {
            var sceneOwner = (this.config)[constant.SCENEOWNER];
            var tenantId = sceneOwner[constant.CONFIG_TENANTID];
            logSummary += ";" + "tenantId" + ":" + tenantId;
            if(sceneOwner.hasOwnProperty(constant.CONFIG_OWNERTYPE)) {
                var ownerType =  sceneOwner[constant.CONFIG_OWNERTYPE];
                logSummary += ";" + "ownerType" + ":" + ownerType;
            }
            if(sceneOwner.hasOwnProperty(constant.CONFIG_OWNERID)) {
                var ownerId = sceneOwner[constant.CONFIG_OWNERID];
                logSummary += ";" + "ownerId" + ":" + ownerId;
            }
        } catch (e) {
            log.LOGE(TAG, "generateLog json err = " + e);
        }
    }
    logSummary += ";" + "bizCode:SUMMARY";
    logSummary += ";" + "traceId" + ":" + this.traceId;
    logSummary += ";" + "sceneId" + ":" + this.sceneid;
    logSummary += ";" + "useTime" + ":" + useTime;
    logSummary += ";" + "result" + ":" + isSuccess;
    logSummary += ";" + "timestamp" + ":" + Math.round(new Date().getTime());// 毫秒
    logSummary += "\n";
    return logSummary;
};

Ifttt.prototype.parseActions = function (actions) {
    if (actions.length === 0) return false;
    // actions.forEach((action)=>{
    //     this.actions.push(new Action(action, this.sceneid, this.proxy));
    // });
    return true;
};

Ifttt.prototype.execute = function (notify, cb) {
    log.LOGI(TAG, "runner_ifttt execute begin.");
    var time = new Date();
    var filepath = this.jobInfo.path;
    var isLegal = false;
    var self = this;
    try {
        fs.readFile(path.join(filepath, constant.jobConfig), {encoding: 'utf-8'}, function (err, data) {
            if (!err) {
                self.config = JSON.parse(data);
                fs.readFile(path.join(filepath, constant.jobIndex), {encoding: 'utf-8'}, function (err, data) {
                    if (err) {
                        self.sendJobStatus(true, err, time, function (err1, status) {
                            return cb(err);
                        });
                    } else {
                        var sceneObj = JSON.parse(data);
                        var isLegal = self.checkFormat(sceneObj);
                        if (notify && !isLegal) {
                            log.LOGE(TAG, "isLegal =" + isLegal);
                            self.sendJobStatus(!isLegal, ExecStatus.format_error, time, function (err, status) {
                                return cb(!isLegal);
                            });
                        }
                        var actionsObject = sceneObj[constant.ACTION];
                        if (notify && !self.parseActions(actionsObject)) {
                            log.LOGE(TAG, "parseActions err =" + JSON.stringify(actionsObject));
                            self.sendJobStatus(true, ExecStatus.format_error, time, function (err, status) {
                                return cb(true, ExecStatus.format_error);
                            });
                        }
                        var action = new Action(actionsObject, self.sceneid, self.proxy, self.traceId);
                        action.execute(function (err, detailsLog) {
                            log.LOGI(TAG, "ifttt execute err = " + err  + " detailsLog " + detailsLog);
                            var result;
                            if (err) {
                                result = ExecStatus.failed;
                            } else {
                                result = ExecStatus.success;
                            }
                            if (notify) {
                                // if(notify && status != constant.NO_RUN) {
                                self.sendJobStatus(err, result, time, detailsLog, function (err1, status) {
                                    cb(err, result)
                                });
                            } else {
                                cb(err, result);
                            }
                        });
                    }
                });

            } else {
                self.sendJobStatus(err, err, time, function (err, status) {
                    return cb(err);
                });
            }
        });
    } catch (e) {
        log.LOGE(TAG, util.format("[ERROR] parse ifttt rule '%s' failed: %s", filepath, e.message));
        log.LOGE(TAG, e.stack);
        if (notify) {
            self.sendJobStatus(e, e, time);
        }
        cb(e);
    }
    // 有异步回调在里面，怎么才能确定是真正的完成了job？
    // if (cb) {
    //     cb(self.jobInfo.jobId, "dont care");
    // }
};


module.exports = Ifttt;
