'use strict';
var _instance = null;//定义初始化_instance
var log = require("../developer/log");
var TAG = "[registry_DataCenter ]";

var DataCenter = function () {
    _instance = this;
    this.deviceList = [];
    this.deviceMap = new Map();
    this.idList = [];
    // this.myDB = new ProfileDB("devices");
    // this.myDB.dropCollection();
    // this.myDB.createCollection();
    this.profileMap = new Map();
};

/**
 * @description 判断是否已经有这个设备
 * @return {bool}
 */
DataCenter.prototype.hasThing = function(id) {
    return this.idList.indexOf(id) >= 0 ? true : false;
};

/**
 * @description 获取实例化设备列表
 * @returns {Array}
 */
DataCenter.prototype.getDeviceObjectList = function () {
    return this.deviceList;
};

/**
 * @description 获取实例化设备ID列表
 * @returns {Array}
 */
DataCenter.prototype.getObjectIDList = function () {
    return this.idList;
};

/**
 * @description 获取id对应的实例化device
 * @param id
 * @returns {DynamicDevice} device对象
 */
DataCenter.prototype.getDeviceObject = function (id) {
    return this.deviceMap.get(id);
};

/**
 * @description 添加设备入口
 * @param {String} id
 * @param {DynamicDevice} device 对象
 **/
DataCenter.prototype.addDevice = function (id, device) {
    if(id == "" || id == null || device == null) {
        log.LOGE(TAG, "addDevice Error Device id = " + id + " device = " + device);
        return;
    }
    if(this.idList.indexOf(id) >= 0) {
        log.LOGI(TAG, "addDevice id has existed, id =" + id);
        return;
    }
    this.deviceList.push(device);
    this.deviceMap.set(id, device);
    this.idList.push(id);
    log.LOGI(TAG, "deviceList num = " + this.deviceList.length);
    log.LOGI(TAG, "addDevice now add id = " + id + " \nDataCenter.idList.size = " + this.idList.length);
    //add profile : js object
    // this.myDB.add(id, profile, function (err, id, result) {
    //     if (!err) {
    //         console.log("add " + id, result);
    //     } else {
    //         console.log("add " + err);
    //     }
    // });
};

/**
 * @description 删除设备入口
 * @param {String} id
 **/
DataCenter.prototype.deleteDevice = function (id) {
    log.LOGI(TAG, "DataCenter devareDeviceId= " + id);
    if (id == "" || id == undefined || id == null) {
        console.log("Error param id = " + id);
        return;
    }
    // var profile = this.profileMap.get(id);
    // this.myDB.remove(profile, function (err, result) {
    //     console.log("err = " + err + " remove result = " + result);
    // });
    var device = this.deviceMap.get(id);
    for (var i in this.deviceList) {
        if (this.deviceList[i].id === id) {
            this.deviceList.splice(i, 1);
            break;
        }
    }
    this.deviceMap.devare(id);
    for (var i in this.idList) {
        if (this.idList[i] === id) {
            this.idList.splice(i, 1);
            break;
        }
    }
    device = null;
};

/**
 * @description 清空所有设备数据列表（慎用)
 *
 */
DataCenter.prototype.clearThing = function () {
    log.LOGI(TAG, "clearThing begin...");
    this.deviceList.length = 0;
    this.idList.length = 0;
    this.deviceMap.clear();
};

DataCenter.prototype.hasProfile = function (productKey) {
    return this.profileMap.has(productKey);
};

DataCenter.prototype.getProfile = function (productKey) {
    return this.profileMap.get(productKey);
};

DataCenter.prototype.addProfile = function (productKey, profile) {
    log.LOGI(TAG, "productKey = " + productKey);
    if( !this.hasProfile(productKey) && profile && profile != "" && profile != "{}") {
        this.profileMap.set(productKey, profile);
    }
    log.LOGI(TAG, "profileMap num = " + this.profileMap.size);
};

DataCenter.prototype.devareProfile = function (productKey) {
    return this.profileMap.devare(productKey);
};

DataCenter.prototype.clearProfile = function () {
    this.profileMap.clear();
};

/**
 * @description 实例化DataCenter对象
 * @returns {DataCenter} 对象的单例
 */
DataCenter.getInstance = function () {
    if (_instance != null) {
        return _instance;
    } else {
        return new DataCenter();
    }
};
module.exports = DataCenter.getInstance();

