#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import sys
import getopt
import importlib
import logging
import traceback
import fc_error
import ws_client
import utils
import config
import ipc_wrapper
import base64
import io
import json


sys.stdout = io.TextIOWrapper(sys.stdout.buffer, encoding='utf8', line_buffering=False, write_through=True)
def gen_err_log(invocationId, message, user_ex = None):
    logging.getLogger().error(traceback.format_exc())
    exc_info = sys.exc_info()
    if not message:
        message = fc_error.get_error_response(user_ex, exc_info[2], True)
    logging.error("fail:%s", message)
    
    if ipc_wrapper.ipc is not None:
        param = ipc_wrapper.ipc.postHandlerErr(invocationId, message)
        if param is not None and "statusCode" in param["msg"]:
            statusCode = param["msg"]["statusCode"]
            if statusCode == 200:
                logging.debug("run_custom_func:succ! result:%s", result)
            else:
                logging.error("send result fail but exec succ! result:%s", result)

def startJob():
    handler = ""
    result = False
    Func = None
    
    argv = sys.argv[1:]
    try:
        opts, args = getopt.getopt(sys.argv[1:], "", ["handler=", "name="])
        for cmd, arg in opts:
            if cmd in ("--handler"): 
                handler = arg
        if handler.find(".", 1, len(handler) - 1) == -1:
            gen_err_log("", "error handler")
        else:
            result, Func = parse_handler(handler)
    except Exception as e:
        gen_err_log("", None, e)
        return fc_error.PY_RUNTIME_ERROR_FAILED
    if result:
        while True:
            try:
                param = ipc_wrapper.ipc.get_param()
                invocationId = ""
                if param is not None and "statusCode" in param["msg"]:
                    statusCode = param["msg"]["statusCode"]
                    if statusCode == 200:
                        invokerContext = None
                        if "headers" in param["msg"]:
                            headers = param["msg"]["headers"]
                            invocationId = headers[config.HEADER_INVOCATION_ID]
                            if config.HEADER_INVOKER_CONTEXT in headers:
                                invokerstr = headers[config.HEADER_INVOKER_CONTEXT]
                                try:
                                    invokerContext = base64.b64decode(invokerstr).decode("utf-8")
                                except Exception as e:
                                    invokerContext = None
                        # get event param
                        if "body" in param["msg"]:
                            event = base64.b64decode(param["msg"]["body"])
                        else:
                            event = ""
                        # get context param    
                        if invokerContext:
                            invokerContextjson = json.loads(invokerContext)
                            custom_value = ''
                            if "custom" in invokerContextjson:
                                custom_value = invokerContextjson["custom"] 
                            context = {
                                "requestId": invocationId,
                                "invokerContext": {
                                    "custom": custom_value}
                            }
                        else:
                            context = {
                                "requestId": invocationId,
                                "invokerContext": {
                                    "custom": {}}
                            }
                    else:
                        body = "get param fail"
                        if "body" in param["msg"]:
                            body = base64.b64decode(param["msg"]["body"]).decode("utf-8")
                        gen_err_log(invocationId, body)
                        continue
                else:
                    gen_err_log(invocationId, "wrong format with get parameter, No statusCode")
                    continue
                result = Func(event, context)
                sys.stdout.flush()
                logging.debug("result:%s", result)
                param = ipc_wrapper.ipc.postTaskResult(invocationId, result)
                if param is not None and "statusCode" in param["msg"]:
                    statusCode = param["msg"]["statusCode"]
                    if statusCode == 200:
                        logging.debug("run_custom_func:succ! result:%s", result)
                    else:
                        logging.error("send result fail but exec succ! result:%s", result)
            except Exception as user_ex:
                gen_err_log("", None, user_ex)
    else:
        gen_err_log("", "can't found user's entry function")

def parse_handler(handler):
    module_name, func_name = handler.rsplit('.', 1)
    module = importlib.import_module(module_name)
    try:
        request_handler = getattr(module, func_name)
    except Exception as user_ex:
        gen_err_log(None, user_ex)
        return False, None
    return True, request_handler

if(__name__ == "__main__"):
    fmt = "<%(filename)s:%(lineno)s> %(levelname)s - %(message)s"
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG, format=fmt)
    startJob()

